/* sdk 版本 1.0.0 */
function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var https = _interopDefault(require('https'));
var crypto = _interopDefault(require('crypto'));

var version = "1.0.0";

const Config = {
    DEBUG: false,
    sdkVersion: version,
    // Hubble平台host
    api_host: 'hubble.netease.com',
    abtest_url: '/cc/a_exp/',
    // 日志上报的路径
    track_url: '/track/s/',
    abtest: {
        enable_abtest: false,
        timeoutMs: 500
    }
};

// 事件类型
// 默认事件类型，自定义
const DATATYPE = 'e';

//内置事件列表
const DEFAULTEVENTID = {
    //表示会话开始事件
    'da_session_start': {
        'dataType': 'ie'
    },
    //表示会话结束事件
    'da_session_close': {
        'dataType': 'ie'
    },
    //通过用户登陆传入的 userId 信息来映射设备 ID, 用户登出事件
    'da_u_login': {
        'dataType': 'ie'
    },
    //用户登录事件
    'da_u_logout': {
        'dataType': 'ie'
    },
    //用户 ID 关联 绑定输入的 newUserId 和已有 userID，用户注册等同一用户 userId 变动场景。
    'da_u_signup': {
        'dataType': 'ie'
    },
    //用户属性设置内部事件
    'da_user_profile': {
        'dataType': 'ie'
    },
    //页面浏览事件，浏览是一大类用户交互集合，设计特定 dataType = “pv”
    'da_screen': {
        'dataType': 'pv'
    },
    //广告点击事件
    'da_ad_click': {
        'dataType': 'ie'
    },
    //应用激活事件，应用第一次打开时发送
    'da_activate': {
        'dataType': 'ie'
    },
    //abtest事件 
    'da_abtest': {
        'dataType': 'ie'
    },
    // 发送数据异常错误
    'da_send_error': {
        'dataType': 'ie'
    }
};
// 用户属性上报数据所属事件id
const PEOPLE_PROPERTY_ID = 'da_user_profile';

// sdk上报数据所属类型
const SDKTYPE = 'server-node';

/**
 * People类
 */

const People = {
    init(DATracker) {
        this.DATracker = DATracker;
        return this;
    },

    // 验证yyyy-MM-dd日期格式
    checkTime(timeString) {
        const date = timeString + '';
        const reg = /^(\d{4})-(\d{2})-(\d{2})$/; 
        if(date) {
          if (!reg.test(date)){
            return false;
          } else {
            return true;
          }
        } else {
            return false;
        }
    },

    /**
     * 上报用户属性
     * @param {String} userId  用户id
     * @param {Object} properties 用户属性
     */
    peopleSet(userId, properties) {
        if (typeof properties == 'undefined' || properties == null) {
            this.DATracker.log('warn', '用户属性不能为空');
            return false;
        }
        let userProfile = {
            '$type': 'profile_set'
        };
        properties = Object.assign({}, userProfile, properties);
        this.DATracker.track(userId, PEOPLE_PROPERTY_ID, {'$userProfile': properties});
    },

    // 设置用户自定义属性
    set(userId, properties) {
        this.peopleSet(userId, properties);
    },
    /**
     * 设置: 姓名
     */
    set_realname(userId, realname) {
        this.peopleSet(userId, {"$realName" : realname});
    },
    /**
     * 设置: 国家
     */
    set_country(userId, country) {
        this.peopleSet(userId, {"$country" : country});
    },
    /**
     * 设置：省份
     */
    set_province(userId, province) {
        this.peopleSet(userId, {"$region" : province});
    },
    /**
     * 设置：城市
     */
    set_city(userId, city) {
        this.peopleSet(userId, {"$city" : city});
    },
    /**
     * 设置：性别  0-女，1-男，2-未知
     */
    set_gender(userId, gender) {
        if ([0,1,2].indexOf(gender) > -1) {
            this.peopleSet(userId, {"$gender" : gender});
        }
    },
    /**
     * 设置：生日
     */
    set_birthday(userId, birthday) {
        if(!this.checkTime(birthday)) return; 
        this.peopleSet(userId, {"$birthday" : birthday});
    },
    /**
     * 大集合：账户 + 姓名 + 生日 + 性别
     */
    set_populationWithAccount(userId, account, realname, birthday, gender) {
        if(!account || !realname || !birthday || [0,1,2].indexOf(gender) === -1) return;
        if(!this.checkTime(birthday)) return; 
        //生日合法检测，yy-MM-dd
        this.peopleSet(userId, {'$account': account, "$realName" : realname, "$birthday": birthday, "$gender": gender});
    },
    /**
     * 国家 + 省份 + 城市
     */
    set_location(userId, country, region, city) {
        if(!country || !region || !city) return;
        this.peopleSet(userId, {"$country" : country, "$region": region, "$city": city});
    }

};

/**
 * abtest模块
 */

const ABTest = {
    init(DATracker) {
        this.DATracker = DATracker;
        this.config = this.DATracker.getConfig('abtest');
        return this;
    },
    // 检测字段为空
    isEmpty(checkStr) {
        let bool = false;
        if (checkStr == null || typeof checkStr == 'undefined' || checkStr == '') {
            bool = true;
        }
        return bool;
    },
    // 上报 abtest 事件
    trackAbtest(userId, properties) {
        this.DATracker.track(userId, 'da_abtest', properties);
    },
    // 返回值，当前变量的实验信息
    variableToFindExperiment(variables, experiments, variableName) {
        let dataObj = {
            $experimentId: [],
            $versionId: [],
            $experimentType: 1
        };
        experiments.map( experiment => {
            // 在返回的实验数据里找到了变量
            if (experiment['variables'].indexOf(variableName) > -1) {
                if (dataObj.$experimentId.indexOf(experiment['experimentId']) === -1) {
                    dataObj.$experimentId.push(experiment['experimentId']);
                }
                if (dataObj.$versionId.indexOf(experiment['versionId']) === -1) {
                    dataObj.$versionId.push(experiment['versionId']);
                }
            }
        } );
        dataObj[variableName] = variables[variableName];
        dataObj.$experimentId = dataObj.$experimentId.join(',');
        dataObj.$versionId = dataObj.$versionId.join(',');

        return dataObj;
    },
    /**
     * 获取实验变量值
     * @param {String} userId 用户
     * @param {String} variableName 实验变量名
     * @param {String} defaultValue 默认变量值
     */
    get_variation(userId, variableName, defaultValue) {
        let proms = new Promise((resolve, reject) => {
            resolve(defaultValue);
        });
        if (this.isEmpty(userId)) {
            this.DATracker.log('warn', "用户id不能为空!");
            return proms;
        }
        if (this.isEmpty(variableName)) {
            this.DATracker.log('warn', "实验变量不能为空!");
            return proms;
        }
        //打开了abtest功能
        if (this.config.enable_abtest) {
            return this.getVariable(userId, variableName, defaultValue);
        } else {
            this.DATracker.log('warn', "请在配置里开启abtest");
        }
        return proms;
    },
    // 远程拉取数据
    getVariable(userId, variableName, defaultValue) {
        let options = {
            hostname: this.DATracker.getConfig('api_host'),
            port: '',
            path: this.DATracker.getConfig('abtest_url'),
            method: 'POST',
            headers: {
                'Content-Type': 'application/json;charset=UTF-8'
            }
        };
        options.agent = new https.Agent({
            timeout: typeof this.config.timeoutMs == 'number' ? this.config.timeoutMs : 500
        });
        let params = {
            appKey: this.DATracker.getConfig('appKey'),
            userId: userId
        };
       
        return new Promise((resolve, reject) => {
           const req =  https.request(options);
           req.on('response', (res) => {
            res.setEncoding('utf8');
            res.on('data', data => {
                try {
                    let value;
                    this.DATracker.log('log', '拉取的实验信息'+ data);
                    data = JSON.parse(data);
                    if (data.code == 200) {
                        let variables = data.data.variables;
                        let experiments = data.data.experiments;
                        let dataObj = this.variableToFindExperiment(variables, experiments, variableName);
                        value = dataObj[variableName];
                        if (typeof value !== 'undefined') {
                            this.trackAbtest(userId, dataObj);
                        } else {
                            value = defaultValue;
                        }
                    } else {
                        value = defaultValue;
                    }
                    resolve(value);
                } catch (error) {
                    resolve(defaultValue);
                }
            });
            res.on('error', e => {
                resolve(defaultValue);
            });
           });
           req.write(JSON.stringify(params));
           req.end();
        });
    }
};

/**
 * 
 */


 // 只能有一个实例
 const DATracker = {
    config: {},
    people: null,
    abtest: null,
    /**
      * 
      * @param {String} appkey 应用appkey
      * @param {Object} config 自定义配置
    */
    init(appkey, config) {
        if (typeof appkey == 'undefined') {
            throw new Error("appkey不能为空");
        }
        const defaultConfig = Object.assign({}, Config, {
            appKey: '',
            ipPattern: '^(([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.){3}([01]?\\d\\d?|2[0-4]\\d|25[0-5])$',
            superProperties: {}
        });
        this.config = Object.assign({}, defaultConfig, config || {});
        
        this.setConfig('appKey', appkey);
        this.people = People.init(this);
        this.abtest = ABTest.init(this);
        return this;
    },
    // type: warn, error, log
    log(type, msg) {
        if (this.getConfig('DEBUG')) {
            if (console[type]) {
                console[type](msg);
            }
        }
    },
    setConfig(name, value) {
        this.config[name] = value;
    },
    getConfig(name) {
        return this.config[name];
    },
    // 发送日志到服务器
    send(properties) {
        const sha1 = (str) => {
           return crypto.createHash('sha1').update(str).digest('hex').toUpperCase();
        };
        let dataStr = null;
        let base64Str = null;
        let encodeStr = null;
        if (typeof properties == null) {
            this.log('error',"上传日志不能为空");
        }
        try {
            dataStr = JSON.stringify(properties);
            base64Str = Buffer.from(dataStr).toString('base64');
            encodeStr = encodeURIComponent(base64Str);
            this.log('log', dataStr);
            let options = {
                hostname: this.getConfig('api_host'),
                port: '',
                path: this.getConfig('track_url') + '?data=' + encodeStr,
                method: 'GET',
                headers: {
                    'X-CLIENT-IP': properties.ip,
                    'X-SHA1-APPKEY': sha1(this.getConfig('appKey'))
                }
            };

            if (!properties.ip) {
                delete options.headers['X-CLIENT-IP'];
            }

            options.agent = new https.Agent({
                keepAlive: true,
                timeout: 300000,
                keepAliveMsecs: 300000,
                // 每个主机允许的套接字的最大数量
                maxSockets: 100,
                //maxFreeSockets: 1000
            });

            let req = https.request(options, (res) => {});
            req.on('response', (res) => {
                if (res.statusCode != 200) {
                    this.log('error', 'Hubble Server 日志发送失败：'+ res.statusCode, dataStr);
                }
            });
            req.on('error', (e) => {
                this.log('warn', 'https request 失败 '+ e, );
            });
            req.end();
        } catch (error) {
            this.log('error', "日志字符串化失败 "+ error);
        }
    },
    // 检测公共属性ip是否合法
    checkIp(ip) {
        let reg = new RegExp(this.getConfig('ipPattern'));
        return reg.test(ip);
    },
    // 检测用户id不能为空
    checkUserId(userId) {
        let bool = true;
        if (userId == null || typeof userId == 'undefined' || userId == '') {
            bool = false;
        }
        return bool;
    },
    // 检测事件id不能为空
    checkEventId(eventId) {
        let bool = true;
        if (eventId == null || typeof eventId == 'undefined' || eventId == '') {
            bool = false;
        }
        return bool;
    },
    // sdk默认事件属性
    getDefaultProperties() {
        return {
            sdkType: SDKTYPE,
            sdkVersion: this.getConfig('sdkVersion'),
            appKey: this.getConfig('appKey')
        };
    },
    /**
     * 上报事件日志（对外）
     * @param {String} userId  
     * @param {String} eventId 事件id 
     * @param {Object} properties 事件自定义属性
     * @param {Object} systemProperties 系统属性
     */
    track(userId, eventId, properties, systemProperties) {
        try {
            if (!this.checkUserId(userId)) {
                this.log('warn', "用户id不能为空");
                return;
            }
            if (!this.checkEventId(eventId)) {
                this.log('warn', "事件id不能为空");
                return;
            }
            // 事件类型
            let dataType = DATATYPE;
            // 如果是内置事件,事件类型重新设置
            if (DEFAULTEVENTID[eventId]) {
                dataType = DEFAULTEVENTID[eventId].dataType;
            }
            properties = typeof properties == 'object' ? properties : {};
            systemProperties = typeof systemProperties == 'object' ? systemProperties : {};
            
            // ip
            let ip = typeof systemProperties.ip == 'string' ? systemProperties.ip : null;
            // 事件耗时
            let costTime = typeof systemProperties.costTime == 'number' ? systemProperties.costTime : null;

            // 系统属性
            let event = {};
            event.dataType = dataType;
            event.time = new Date().getTime();
            event.userId = userId;
            event.eventId = eventId;
            
            if (costTime != null) {
                event.costTime = costTime;
            }
            if (ip != null) {
                event.ip = ip;
                if (!this.checkIp(ip)) {
                    this.log('warn', 'ip格式不对');
                }
            }
            // 合并公共用户自定义事件属性，排除用户事件
            if (eventId != PEOPLE_PROPERTY_ID) {
                let superProperties = this.getConfig('superProperties') || {};
                properties = Object.assign({}, superProperties, properties);
            }
            // 自定义属性
            event.attributes = properties;
            event = Object.assign({}, this.getDefaultProperties(), event);
            this.send(event);
        } catch (error) {
            this.log('error', error);
        }
    },
    // 用户行为事件的通用属性设置
    register_attributes(properties) {
        let superProperties = this.getConfig('superProperties') || {};
        superProperties = Object.assign({}, superProperties, properties || {});
        this.setConfig('superProperties', superProperties);
    },
    // 用户行为事件的通用属性设置（不覆盖之前的）
    register_attributes_once(properties) {
        let superProperties = this.getConfig('superProperties') || {};
        superProperties = Object.assign({}, properties || {}, superProperties);
        this.setConfig('superProperties', superProperties);
    },
    // 查看当前已设置的用户行为事件的通用属性
    current_attributes() {
        return this.getConfig('superProperties') || {};
    },
    // 删除一个用户行为事件的通用属性
    unregister_attributes(propertyName) {
        if (typeof propertyName === 'string') {
            let superProperties = this.getConfig('superProperties') || {};
            delete superProperties[propertyName];
            this.setConfig({'superProperties': superProperties});
        }
    },
    // 删除所有已设置的用户行为事件的通用属性
    clear_attributes() {
        this.setConfig('superProperties', {});
    },
    // 用户注册，把匿名ID和注册ID贯通起来
    // oldUserId: 用户匿名id
    // newUserId: 用户id
    signup(oldUserId, newUserId) {
        if (this.checkUserId(oldUserId) &&  this.checkUserId(newUserId)) {
            let properties = {
                'oldUserId': oldUserId,
                'newUserId': newUserId
            };
            this.track(newUserId, 'da_u_signup', properties);
        } else {
            this.log('warn', 'signup方法：两个参数不能为空！');
        }
    }
 };

module.exports = DATracker;
/* 关注Hubble平台:https://hubble.netease.com; 有问题请联系我：hzwangyang1@corp.netease.com  */
