/**
 * abtest模块
 */
import https from 'https';

const ABTest = {
    init(DATracker) {
        this.DATracker = DATracker;
        this.config = this.DATracker.getConfig('abtest');
        return this;
    },
    // 检测字段为空
    isEmpty(checkStr) {
        let bool = false;
        if (checkStr == null || typeof checkStr == 'undefined' || checkStr == '') {
            bool = true;
        }
        return bool;
    },
    // 上报 abtest 事件
    trackAbtest(userId, properties) {
        this.DATracker.track(userId, 'da_abtest', properties);
    },
    // 返回值，当前变量的实验信息
    variableToFindExperiment(variables, experiments, variableName) {
        let dataObj = {
            $experimentId: [],
            $versionId: [],
            $experimentType: 1
        };
        experiments.map( experiment => {
            // 在返回的实验数据里找到了变量
            if (experiment['variables'].indexOf(variableName) > -1) {
                if (dataObj.$experimentId.indexOf(experiment['experimentId']) === -1) {
                    dataObj.$experimentId.push(experiment['experimentId']);
                }
                if (dataObj.$versionId.indexOf(experiment['versionId']) === -1) {
                    dataObj.$versionId.push(experiment['versionId']);
                }
            }
        } );
        dataObj[variableName] = variables[variableName];
        dataObj.$experimentId = dataObj.$experimentId.join(',');
        dataObj.$versionId = dataObj.$versionId.join(',');

        return dataObj;
    },
    /**
     * 获取实验变量值
     * @param {String} userId 用户
     * @param {String} variableName 实验变量名
     * @param {String} defaultValue 默认变量值
     */
    get_variation(userId, variableName, defaultValue) {
        let proms = new Promise((resolve, reject) => {
            resolve(defaultValue);
        });
        if (this.isEmpty(userId)) {
            this.DATracker.log('warn', "用户id不能为空!");
            return proms;
        }
        if (this.isEmpty(variableName)) {
            this.DATracker.log('warn', "实验变量不能为空!");
            return proms;
        }
        //打开了abtest功能
        if (this.config.enable_abtest) {
            return this.getVariable(userId, variableName, defaultValue);
        } else {
            this.DATracker.log('warn', "请在配置里开启abtest");
        }
        return proms;
    },
    // 远程拉取数据
    getVariable(userId, variableName, defaultValue) {
        let options = {
            hostname: this.DATracker.getConfig('api_host'),
            port: '',
            path: this.DATracker.getConfig('abtest_url'),
            method: 'POST',
            headers: {
                'Content-Type': 'application/json;charset=UTF-8'
            }
        };
        options.agent = new https.Agent({
            timeout: typeof this.config.timeoutMs == 'number' ? this.config.timeoutMs : 500
        });
        let params = {
            appKey: this.DATracker.getConfig('appKey'),
            userId: userId
        };
       
        return new Promise((resolve, reject) => {
           const req =  https.request(options);
           req.on('response', (res) => {
            res.setEncoding('utf8');
            res.on('data', data => {
                try {
                    let value;
                    this.DATracker.log('log', '拉取的实验信息'+ data);
                    data = JSON.parse(data);
                    if (data.code == 200) {
                        let variables = data.data.variables;
                        let experiments = data.data.experiments;
                        let dataObj = this.variableToFindExperiment(variables, experiments, variableName);
                        value = dataObj[variableName];
                        if (typeof value !== 'undefined') {
                            this.trackAbtest(userId, dataObj);
                        } else {
                            value = defaultValue;
                        }
                    } else {
                        value = defaultValue;
                    }
                    resolve(value);
                } catch (error) {
                    resolve(defaultValue);
                }
            });
            res.on('error', e => {
                resolve(defaultValue);
            });
           });
           req.write(JSON.stringify(params));
           req.end();
        });
    }
};

export default ABTest;