/**
 * People类
 */

import {PEOPLE_PROPERTY_ID} from './config';

const People = {
    init(DATracker) {
        this.DATracker = DATracker;
        return this;
    },

    // 验证yyyy-MM-dd日期格式
    checkTime(timeString) {
        const date = timeString + '';
        const reg = /^(\d{4})-(\d{2})-(\d{2})$/; 
        if(date) {
          if (!reg.test(date)){
            return false;
          } else {
            return true;
          }
        } else {
            return false;
        }
    },

    /**
     * 上报用户属性
     * @param {String} userId  用户id
     * @param {Object} properties 用户属性
     */
    peopleSet(userId, properties) {
        if (typeof properties == 'undefined' || properties == null) {
            this.DATracker.log('warn', '用户属性不能为空');
            return false;
        }
        let userProfile = {
            '$type': 'profile_set'
        };
        properties = Object.assign({}, userProfile, properties);
        this.DATracker.track(userId, PEOPLE_PROPERTY_ID, {'$userProfile': properties});
    },

    // 设置用户自定义属性
    set(userId, properties) {
        this.peopleSet(userId, properties);
    },
    /**
     * 设置: 姓名
     */
    set_realname(userId, realname) {
        this.peopleSet(userId, {"$realName" : realname});
    },
    /**
     * 设置: 国家
     */
    set_country(userId, country) {
        this.peopleSet(userId, {"$country" : country});
    },
    /**
     * 设置：省份
     */
    set_province(userId, province) {
        this.peopleSet(userId, {"$region" : province});
    },
    /**
     * 设置：城市
     */
    set_city(userId, city) {
        this.peopleSet(userId, {"$city" : city});
    },
    /**
     * 设置：性别  0-女，1-男，2-未知
     */
    set_gender(userId, gender) {
        if ([0,1,2].indexOf(gender) > -1) {
            this.peopleSet(userId, {"$gender" : gender});
        }
    },
    /**
     * 设置：生日
     */
    set_birthday(userId, birthday) {
        if(!this.checkTime(birthday)) return; 
        this.peopleSet(userId, {"$birthday" : birthday});
    },
    /**
     * 大集合：账户 + 姓名 + 生日 + 性别
     */
    set_populationWithAccount(userId, account, realname, birthday, gender) {
        if(!account || !realname || !birthday || [0,1,2].indexOf(gender) === -1) return;
        if(!this.checkTime(birthday)) return; 
        //生日合法检测，yy-MM-dd
        this.peopleSet(userId, {'$account': account, "$realName" : realname, "$birthday": birthday, "$gender": gender});
    },
    /**
     * 国家 + 省份 + 城市
     */
    set_location(userId, country, region, city) {
        if(!country || !region || !city) return;
        this.peopleSet(userId, {"$country" : country, "$region": region, "$city": city});
    }

}; 


export default People;
