/**
 * 测试代码
 */


const chai = require('chai');
const sinon = require('sinon');
const expect = chai.expect;

const packageJson = require('../package.json');

const hubbleSdk = require('../lib/hubble-sdk-node');


const appKey = 'MA-86F6-2C22AE1EB389';
const anonymousUser = 'zz-xxx-ffff-kk';
const userId = 'hzwangyang1-test';
const eventId = 'haha';


describe("sdk", () => {
    let sandbox, clock;
    beforeEach(() => {
        sandbox = sinon.createSandbox();
        // 延迟时钟
        clock = sinon.useFakeTimers();
        // 延迟一段时间执行(ms)
        //clock.tick(650)
        // 说明：每次重新执行init方法，sdk就重置。
        hubbleSdk.init(appKey);
    });
    afterEach(() => {
        clock.restore();
        sandbox.restore();
    });
    


    describe("内部方法：设置属性API- setConfig", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.setConfig).to.be.equal('function');
        });
        it("随意设置一个属性test, 值为as2", () => {
            hubbleSdk.setConfig('test', 'as2');
            expect(hubbleSdk.config.test).to.be.equal('as2');
        });
    });
    describe("内部方法：获取属性API- getConfig", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.getConfig).to.be.equal('function');
        });
        it("获取一个属性test，值为as2", () => {
            hubbleSdk.setConfig('test', 'as2');
            let test = hubbleSdk.getConfig('test');
            expect(test).to.be.equal('as2');
        });
    });
    describe("内部方法：检测ip API- checkIp", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.checkIp).to.be.equal('function');
        });
        it("传入ip：192.168.0.1，返回了一个boolean为true", () => {
            let bool = hubbleSdk.checkIp('192.168.0.1');
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(true);
        });
        it("传入不合法ip：xxx，返回了一个boolean为false", () => {
            let bool = hubbleSdk.checkIp('xxx');
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(false);
        });
    });
    describe("内部方法：检测用户id API- checkUserId", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.checkUserId).to.be.equal('function');
        });
        it("传入用户id：白云飘飘，返回了一个boolean为true", () => {
            let bool = hubbleSdk.checkUserId('白云飘飘');
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(true);
        });
        it("传入空的用户id：''，返回了一个boolean为false", () => {
            let bool = hubbleSdk.checkUserId('');
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(false);
        });
        it("不传入用户id，返回了一个boolean为false", () => {
            let bool = hubbleSdk.checkUserId();
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(false);
        });
    });
    describe("内部方法：检测事件id API- checkEventId", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.checkEventId).to.be.equal('function');
        });
        it("传入事件id：haha，返回了一个boolean为true", () => {
            let bool = hubbleSdk.checkEventId('haha');
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(true);
        });
        it("传入空的事件id：''，返回了一个boolean为false", () => {
            let bool = hubbleSdk.checkEventId('');
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(false);
        });
        it("不传入事件id，返回了一个boolean为false", () => {
            let bool = hubbleSdk.checkEventId();
            expect(typeof bool).to.be.equal('boolean');
            expect(bool).to.be.equal(false);
        });
    });
    describe("内部方法：获取默认事件属性 API- getDefaultProperties", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.getDefaultProperties).to.be.equal('function');
        });
        it("返回的结果: sdkType, sdkVersion, appKey", () => {
            let obj = hubbleSdk.getDefaultProperties();
            expect(obj.sdkType).to.be.equal('server-node');
            expect(obj.sdkVersion).to.be.equal(packageJson.version);
            expect(obj.appKey).to.be.equal(appKey);
        });
    });
    describe("外部方法：用户行为事件的通用属性设置 API- register_attributes", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.register_attributes).to.be.equal('function');
        });
        it("随意设置一个属性testAll, 值为all，此时发送的日志里包含该属性", () => {
            hubbleSdk.register_attributes({testAll: 'all'});
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.send = sandbox.stub(hubbleSdk, 'send');
            // 模拟上报一个日志
            hubbleSdk.track(anonymousUser, eventId);
            const attributes = hubbleSdk.send.args[0][0]['attributes'];

            expect(attributes['testAll']).to.be.equal('all');
        });
    });
    describe("外部方法：用户行为事件的通用属性设置（不覆盖之前的） API- register_attributes_once", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.register_attributes_once).to.be.equal('function');
        });
        it("设置一个属性testAll-1, 值为all-1，此时发送的日志里包含该属性", () => {
            hubbleSdk.register_attributes_once({'testAll-1': 'all-1'});
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.send = sandbox.stub(hubbleSdk, 'send');
            // 模拟上报一个日志
            hubbleSdk.track(anonymousUser, eventId);
            const attributes = hubbleSdk.send.args[0][0]['attributes'];

            expect(attributes['testAll-1']).to.be.equal('all-1');
        });
        it("重置一个属性testAll-1, 值传入为all-2，此时发送的日志里值不等于 all-2", () => {
            hubbleSdk.register_attributes_once({'testAll-1': 'all-1'});
            hubbleSdk.register_attributes_once({'testAll-1': 'all-2'});
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.send = sandbox.stub(hubbleSdk, 'send');
            // 模拟上报一个日志
            hubbleSdk.track(anonymousUser, eventId);
            const attributes = hubbleSdk.send.args[0][0]['attributes'];
            expect(attributes['testAll-1']).not.to.be.equal('all-2');
        });
        it("重置一个属性testAll-2, 值传入为all-2，此时发送的日志里值不等于 all-2", () => {
            hubbleSdk.register_attributes({'testAll-2': 'all-1'});
            hubbleSdk.register_attributes_once({'testAll-2': 'all-2'});
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.send = sandbox.stub(hubbleSdk, 'send');
            // 模拟上报一个日志
            hubbleSdk.track(anonymousUser, eventId);
            const attributes = hubbleSdk.send.args[0][0]['attributes'];

            expect(attributes['testAll-2']).not.to.be.equal('all-2');
        });
    });
    describe("外部方法：用户行为事件的通用属性设置 API- current_attributes", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.current_attributes).to.be.equal('function');
        });
        // TODO
        it("查看结果为空", () => {
            const attributes = hubbleSdk.current_attributes();
            expect(Object.getOwnPropertyNames(attributes).length == 0).to.be.true;
        });
    });
    describe("外部方法：删除所有已设置的用户行为事件的通用属性 API- clear_attributes", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.clear_attributes).to.be.equal('function');
        });
        it("调用方法清空", () => {
            hubbleSdk.register_attributes({ff:'cc'});
            hubbleSdk.register_attributes({kk:'cc'});
            hubbleSdk.clear_attributes();
            const attributes = hubbleSdk.current_attributes();
            expect(Object.getOwnPropertyNames(attributes).length == 0).to.be.true;
        });
    });
    describe("外部方法：删除一个用户行为事件的通用属性 API- unregister_attributes", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.unregister_attributes).to.be.equal('function');
        });
        it("调用方法清空某个属性", () => {
            hubbleSdk.register_attributes({gg:'cc'});
            hubbleSdk.unregister_attributes('gg');
            const attributes = hubbleSdk.current_attributes();
            const gg = attributes['gg'];
            expect(typeof gg == 'undefined').to.be.true;
        });
    });
    describe("外部方法：用户注册，把匿名ID和注册ID贯通起来 API- signup", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.signup).to.be.equal('function');
        });
        it("调用方法，上报日志里：userId为新用户, 属性包含老用户和新用户，事件为 da_u_signup", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.send = sandbox.stub(hubbleSdk, 'send');
            // 模拟上报一个日志
            hubbleSdk.signup(anonymousUser, userId);
            const event = hubbleSdk.send.args[0][0];
            const attributes = event['attributes'];

            expect(event.userId == userId).to.be.true;
            expect(event.eventId == 'da_u_signup').to.be.true;
            expect(attributes['oldUserId'] == anonymousUser).to.be.true;
            expect(attributes['newUserId'] == userId).to.be.true;
        });
    });
    describe("外部方法：上报事件日志 API- track", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.track).to.be.equal('function');
        });
        it("调用方法，上报一个自定义事件", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.send = sandbox.stub(hubbleSdk, 'send');
            // 加监听
            sandbox.spy(hubbleSdk.send);
            // 模拟上报一个日志
            hubbleSdk.track(userId, eventId, {ff:'测试'});
            // 监听send方法调用了一次
            expect(hubbleSdk.send.calledOnce).to.be.true;

            // 检查send方法参数
            const event = hubbleSdk.send.args[0][0];
            const attributes = event['attributes'];

            expect(event.userId == userId).to.be.true;
            expect(event.eventId == eventId).to.be.true;
            expect(attributes['ff'] == '测试').to.be.true;
        });
    });
    describe("内部方法：发送日志到服务器 API- send", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.send).to.be.equal('function');
        });
        it("调用方法，上报一个自定义事件，发送日志到服务器", () => {
            // 加监听
            sandbox.spy(hubbleSdk, 'send');
            // 模拟上报一个日志
            hubbleSdk.track(userId, eventId, {ff:'测试'});
            // 监听send方法调用了一次
            expect(hubbleSdk.send.calledOnce).to.be.true;
        });
    });


    // People 类   --  hubbleSdk.people
    describe("内部方法：验证yyyy-MM-dd日期格式 API- checkTime", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.checkTime).to.be.equal('function');
        });
        it("传入时间: 1999-08-03， 返回true", () => {
            expect(hubbleSdk.people.checkTime("1999-08-03")).to.be.true;
        });
        it("传入时间: 1999/08/03， 返回false", () => {
            expect(hubbleSdk.people.checkTime("1999/08/03")).to.be.false;
        });
    });
    describe("内部方法：上报用户属性 API- peopleSet", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.peopleSet).to.be.equal('function');
        });
        it("触发一个用户属性日志上报", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.DATracker.track = sandbox.stub(hubbleSdk.people.DATracker, 'track');
            // 加监听
            sandbox.spy(hubbleSdk.people.DATracker.track);
            // 模拟调用上报
            hubbleSdk.people.peopleSet(userId, {'属性1':'测试1'});
            // 监听track方法调用了一次
            expect(hubbleSdk.people.DATracker.track.calledOnce).to.be.true;
            // 检查track方法参数
            const args = hubbleSdk.people.DATracker.track.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置事件id
            expect(args[1] == 'da_user_profile').to.be.true;
            // 内置属性
            expect(args[2]['$userProfile']['$type'] == 'profile_set').to.be.true;
            // 自定义属性
            expect(args[2]['$userProfile']['属性1'] == '测试1').to.be.true;
            
        });
        it("不传属性，返回false- (说明：必须传用户属性)", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.DATracker.track = sandbox.stub(hubbleSdk.people.DATracker, 'track');
            // 加监听
            sandbox.spy(hubbleSdk.people.DATracker.track);
            let bool = hubbleSdk.people.peopleSet(userId);
            // 此时track不会调用
            expect(hubbleSdk.people.DATracker.track.calledOnce).to.be.false;
            // 属性参数必须传
            expect(bool).to.be.false;
        });
    });
    describe("外部方法：设置用户自定义属性 API- set", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set).to.be.equal('function');
        });
        it("设置一个用户自定义属性", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set(userId, {'属性1':'测试1'});
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 自定义属性
            expect(args[1]['属性1'] == '测试1').to.be.true;
        });
    });
    describe("外部方法：设置用户内置属性：姓名 API- set_realname", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_realname).to.be.equal('function');
        });
        it("设置用户的内置属性：姓名", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_realname(userId, "汪洋");
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$realName'] == '汪洋').to.be.true;
        });
    });
    describe("外部方法：设置用户内置属性：国家 API- set_country", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_country).to.be.equal('function');
        });
        it("设置用户的内置属性：国家", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_country(userId, "中国");
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$country'] == '中国').to.be.true;
        });
    });
    describe("外部方法：设置用户内置属性：省份 API- set_province", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_province).to.be.equal('function');
        });
        it("设置用户的内置属性：省份", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_province(userId, "浙江省");
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$region'] == '浙江省').to.be.true;
        });
    });
    describe("外部方法：设置用户内置属性：城市 API- set_city", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_city).to.be.equal('function');
        });
        it("设置用户的内置属性：城市", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_city(userId, "杭州市");
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$city'] == '杭州市').to.be.true;
        });
    });
    describe("外部方法：设置用户内置属性：性别  0-女，1-男，2-未知 API- set_gender", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_gender).to.be.equal('function');
        });
        it("设置用户的内置属性：性别：女", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            //------------------------------------------------------------
            // 模拟调用上报：设置了性别：女
            hubbleSdk.people.set_gender(userId, 0);
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$gender'] == 0).to.be.true;
        });
        it("设置用户的内置属性：性别：男", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            //------------------------------------------------------------
            // 模拟调用上报：设置了性别：男
            hubbleSdk.people.set_gender(userId, 1);
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$gender'] == 1).to.be.true;
        });
        it("设置用户的内置属性：性别：未知", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            //------------------------------------------------------------
            // 模拟调用上报：设置了性别：未知
            hubbleSdk.people.set_gender(userId, 2);
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$gender'] == 2).to.be.true;
        });
        it("设置用户的内置属性：性别：其它（此时不触发上报）", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            //------------------------------------------------------------
            // 模拟调用上报：设置了性别：传入了其它值(不是0,1,2)
            hubbleSdk.people.set_gender(userId, 5);
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
    });
    describe("外部方法：设置用户内置属性：生日 API- set_birthday", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_birthday).to.be.equal('function');
        });
        it("设置用户的内置属性：生日 (合法的值：1999-08-03)", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_birthday(userId, "1999-08-03");
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$birthday'] == "1999-08-03").to.be.true;
        });
        it("设置用户的内置属性：生日（不合法的值：1999/08/03", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_birthday(userId, "1999/08/03");
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
    });
    describe("外部方法：内置属性大集合：账户 + 姓名 + 生日 + 性别 API- set_populationWithAccount", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_populationWithAccount).to.be.equal('function');
        });
        it("设置用户的内置属性： 传入合法的值：hzwnagyang1 、汪洋、 1999-08-03、1", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_populationWithAccount(userId, "hzwnagyang1", "汪洋","1999-08-03", 1);
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$account'] == "hzwnagyang1").to.be.true;
            expect(args[1]['$realName'] == "汪洋").to.be.true;
            expect(args[1]['$birthday'] == "1999-08-03").to.be.true;
            expect(args[1]['$gender'] == 1).to.be.true;
        });
        it("设置用户的内置属性：传入不合法的值：'' 、汪洋、 1999-08-03、1", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "", "汪洋","1999-08-03", 1);
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
        it("设置用户的内置属性：传入不合法的值：'hzwangyang1' 、''、 1999-08-03、1", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "hzwangyang1", "","1999-08-03", 1);
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
        it("设置用户的内置属性：传入不合法的值：'hzwangyang1' 、'汪洋'、 1999/08/03、1", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "hzwangyang1", "汪洋","1999/08/03", 1);
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
        it("设置用户的内置属性：传入不合法的值：'hzwangyang1' 、'汪洋'、 1999-08-03、5", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "hzwangyang1", "汪洋","1999-08-03", 5);
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
    });
    describe("外部方法：内置属性大集合：国家 + 省份 + 城市 API- set_location", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.people.set_location).to.be.equal('function');
        });
        it("设置用户的内置属性： 传入合法的值：中国 、浙江省、杭州市", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报
            hubbleSdk.people.set_location(userId, "中国", "浙江省","杭州市");
            // 监听peopleSet方法调用了一次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.true;
            // 检查peopleSet方法参数
            const args = hubbleSdk.people.peopleSet.args[0];
            // 用户id
            expect(args[0] == userId).to.be.true;
            // 内置属性
            expect(args[1]['$country'] == "中国").to.be.true;
            expect(args[1]['$region'] == "浙江省").to.be.true;
            expect(args[1]['$city'] == "杭州市").to.be.true;
        });
        it("设置用户的内置属性：传入不合法的值：'' 、浙江省、 杭州市", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "", "浙江省","杭州市");
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
        it("设置用户的内置属性：传入不合法的值：中国 、''、 杭州市", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "中国", "","杭州市");
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
        it("设置用户的内置属性：传入不合法的值：中国 、浙江省、 ''", () => {
            // 重新定义send方法，获取此时上报的日志
            hubbleSdk.people.peopleSet = sandbox.stub(hubbleSdk.people, 'peopleSet');
            // 加监听
            sandbox.spy(hubbleSdk.people.peopleSet);
            // 模拟调用上报,此时不会触发上报
            hubbleSdk.people.set_populationWithAccount(userId, "中国", "浙江省","");
            // 监听peopleSet方法调用了0次
            expect(hubbleSdk.people.peopleSet.calledOnce).to.be.false;
        });
    });

    // Abtest类  -- hubbleSdk.abtest
    describe("内部方法：验证是否为空，为空返回true API- isEmpty", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.abtest.isEmpty).to.be.equal('function');
        });
        it("传入: 'test123'， 返回false", () => {
            expect(hubbleSdk.abtest.isEmpty("test123")).to.be.false;
        });
        it("传入: ''， 返回true", () => {
            expect(hubbleSdk.abtest.isEmpty("")).to.be.true;
        });
        it("传入: ， 返回true", () => {
            expect(hubbleSdk.abtest.isEmpty()).to.be.true;
        });
        it("传入: undefined， 返回true", () => {
            let abe;
            expect(hubbleSdk.abtest.isEmpty(abe)).to.be.true;
        });
    });
    describe("内部方法：上报abtest内置事件 API- trackAbtest", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.abtest.trackAbtest).to.be.equal('function');
        });
        it("调用方法，上报参数里：包含 userId, da_abtest，properties", () => {
            // 重新定义track方法，获取此时上报的参数
            hubbleSdk.abtest.DATracker.track = sandbox.stub(hubbleSdk.abtest.DATracker, 'track');
            // 模拟上报一个日志
            hubbleSdk.abtest.trackAbtest(userId, {dd:'test'});
            const args = hubbleSdk.abtest.DATracker.track.args[0];

            expect(args[0] == userId).to.be.true;
            expect(args[1] == 'da_abtest').to.be.true;
            expect(args[2]['dd'] == 'test').to.be.true;
        });
    });
    describe("内部方法：返回当前变量的实验信息 API- variableToFindExperiment", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.abtest.variableToFindExperiment).to.be.equal('function');
        });
        it("调用方法", () => {
           let data = {"experiments":[{"experimentId":"363","versionId":"813","variables":["bgcolor"]}],"variables":{"bgcolor":"红色"}};
           let obj = hubbleSdk.abtest.variableToFindExperiment(data['variables'], data['experiments'], "bgcolor");

           expect(obj.$experimentId == '363').to.be.true;
           expect(obj.$versionId == '813').to.be.true;
           expect(obj.$experimentType == 1).to.be.true;
           expect(obj.bgcolor == '红色').to.be.true;
        });
    });
    describe("外部方法：获取实验变量值 API- get_variation", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.abtest.get_variation).to.be.equal('function');
        });
        it("调用方法", () => {
            hubbleSdk.init(appKey, {abtest: {
                enable_abtest: true
            }});
            hubbleSdk.abtest.getVariable = sandbox.stub(hubbleSdk.abtest, 'getVariable').callsFake(() => {
                return new Promise((resolve, reject) => {
                    resolve('红色');
                });
            });
            sandbox.spy(hubbleSdk.abtest.getVariable);
            // clock.tick(10);

            hubbleSdk.abtest.get_variation('hzwangyang1-test', "bgcolor", "黑色").then(data => {
               expect(data == '红色').to.be.true;
            });

            expect(hubbleSdk.abtest.getVariable.calledOnce).to.be.true;
        });
        it("未开启abtest，调用方法", () => {
            hubbleSdk.init(appKey, {abtest: {
                enable_abtest: false
            }});
            hubbleSdk.abtest.getVariable = sandbox.stub(hubbleSdk.abtest, 'getVariable').callsFake(() => {
                return new Promise((resolve, reject) => {
                    resolve('红色');
                });
            });
            sandbox.spy(hubbleSdk.abtest.getVariable);

            hubbleSdk.abtest.get_variation('hzwangyang1-test', "bgcolor", "黑色").then(data => {
               expect(data == '黑色').to.be.true;
            });

            expect(hubbleSdk.abtest.getVariable.calledOnce).to.be.false;
        });
    });
    describe("内部方法：远程拉取数据 API- getVariable", () => {
        it("定义了该方法", () => {
            expect(typeof hubbleSdk.abtest.getVariable).to.be.equal('function');
        });
        // 
        it("调用方法", () => {
            hubbleSdk.init(appKey, {abtest: {
                enable_abtest: true
            }});
            // 重新定义   returns
            const t = hubbleSdk.abtest.variableToFindExperiment;
            hubbleSdk.abtest.variableToFindExperiment = sandbox.spy(() => {
                return {
                    $experimentId: '363',
                    $versionId: "813",
                    $experimentType: 1,
                    bgcolor: '红色333'
                  };
            });

            hubbleSdk.abtest.trackAbtest = sandbox.spy(sandbox.stub(hubbleSdk.abtest, 'trackAbtest'));

            sandbox.spy(hubbleSdk.abtest.variableToFindExperiment);

            hubbleSdk.abtest.getVariable('hzwangyang1-test', "bgcolor", "黑色").then(data => {
               expect(data == '红色333').to.be.true;
            });

            //clock.tick(1000);
            setTimeout(() => {
                expect(hubbleSdk.abtest.variableToFindExperiment.calledOnce).to.be.true;
                expect(hubbleSdk.abtest.trackAbtest.calledOnce).to.be.true;

                hubbleSdk.abtest.variableToFindExperiment = t;
            }, 1000); 
            
        });
    });


});
